use wccg_models::event::ActivityApproval;
use wread_mongodb::crud_repository;
use wread_mongodb::mongodb::bson::{doc, oid::ObjectId};
use wread_mongodb::mongodb::error::Error;
use wread_mongodb::mongodb::results::{DeleteResult, InsertOneResult};
use wread_mongodb::mongodb::Database;

const COLLECTION_NAME: &str = "activityApprovals";

pub async fn get(id: &str, db: &Database) -> Result<Option<ActivityApproval>, Error> {
    let id_object = ObjectId::with_string(id).unwrap();
    crud_repository::find_by_id(&id_object, COLLECTION_NAME, &db).await
}

pub async fn get_by_rider_id(
    rider_id: &str,
    db: &Database,
) -> Result<Vec<ActivityApproval>, Error> {
    crud_repository::find(doc! {"riderId": rider_id }, COLLECTION_NAME, &db).await
}

pub async fn get_by_rider_id_for_event(
    rider_id: &str,
    event_id: &str,
    db: &Database,
) -> Result<Vec<ActivityApproval>, Error> {
    crud_repository::find(
        doc! {"riderId": rider_id, "eventId": event_id },
        COLLECTION_NAME,
        &db,
    )
    .await
}

pub async fn get_by_rider_id_for_registration(
    rider_id: &str,
    event_id: &str,
    registration_option_id: &str,
    db: &Database,
) -> Result<Vec<ActivityApproval>, Error> {
    crud_repository::find(doc! {"riderId": rider_id, "eventId": event_id, "registrationOptionId": registration_option_id }, COLLECTION_NAME, &db).await
}

pub async fn get_by_rider_id_for_activity(
    rider_id: &str,
    event_id: &str,
    registration_option_id: &str,
    activity_id: &ObjectId,
    db: &Database,
) -> Result<Option<ActivityApproval>, Error> {
    crud_repository::find_one(doc! {"riderId": rider_id, "eventId": event_id, "registrationOptionId": registration_option_id, "activityId": activity_id }, COLLECTION_NAME, &db).await
}

pub async fn add(
    activity_approval: &ActivityApproval,
    db: &Database,
) -> Result<InsertOneResult, Error> {
    crud_repository::add(activity_approval, COLLECTION_NAME, db).await
}

pub async fn delete(id: &str, db: &Database) -> Result<DeleteResult, Error> {
    let id_object = ObjectId::with_string(id).unwrap();
    let filter = doc! {"_id": id_object };
    crud_repository::delete_one(filter, None, COLLECTION_NAME, &db).await
}
