use wccg_models::order::{PaymentOrder, PaymentStatus};
use wread_mongodb::crud_repository;
use wread_mongodb::mongodb::bson::doc;
use wread_mongodb::mongodb::error::Error;
use wread_mongodb::mongodb::results::{DeleteResult, InsertOneResult, UpdateResult};
use wread_mongodb::mongodb::Database;

const COLLECTION_NAME: &str = "paymentorders";

pub async fn get(id: &str, db: &Database) -> Result<Option<PaymentOrder>, Error> {
    crud_repository::find_by_string_id(&id, COLLECTION_NAME, &db).await
}

pub async fn get_by_rider_id(rider_id: &str, db: &Database) -> Result<Vec<PaymentOrder>, Error> {
    crud_repository::find(doc! {"riderId": rider_id }, COLLECTION_NAME, &db).await
}

pub async fn get_by_rider_id_for_event(
    rider_id: &str,
    event_id: &str,
    db: &Database,
) -> Result<Vec<PaymentOrder>, Error> {
    crud_repository::find(
        doc! {"riderId": rider_id, "eventId": event_id },
        COLLECTION_NAME,
        &db,
    )
    .await
}

pub async fn get_by_rider_id_for_registration(
    rider_id: &str,
    event_id: &str,
    registration_option_id: &str,
    db: &Database,
) -> Result<Vec<PaymentOrder>, Error> {
    crud_repository::find(
        doc! {"riderId": rider_id, "eventId": event_id, "selectedOption": registration_option_id },
        COLLECTION_NAME,
        &db,
    )
    .await
}

pub async fn get_by_rider_id_for_event_payment_success(
    rider_id: &str,
    event_id: &str,
    db: &Database,
) -> Result<Option<PaymentOrder>, Error> {
    crud_repository::find_one(
        doc! {"riderId": rider_id, "eventId": event_id, "paymentStatus": "Success" },
        COLLECTION_NAME,
        &db,
    )
    .await
}

pub async fn get_by_rider_id_for_registration_payment_success(
    rider_id: &str,
    event_id: &str,
    registration_option_id: &str,
    db: &Database,
) -> Result<Option<PaymentOrder>, Error> {
    crud_repository::find_one(
        doc! {"riderId": rider_id, "eventId": event_id, "selectedOption": registration_option_id, "paymentStatus": "Success" },
        COLLECTION_NAME,
        &db,
    )
    .await
}

pub async fn add(payment_order: &PaymentOrder, db: &Database) -> Result<InsertOneResult, Error> {
    crud_repository::add(payment_order, COLLECTION_NAME, db).await
}

pub async fn delete(id: &str, db: &Database) -> Result<DeleteResult, Error> {
    let filter = doc! {"_id": id };
    crud_repository::delete_one(filter, None, COLLECTION_NAME, &db).await
}

pub async fn update_payment_status(
    id: &str,
    status: PaymentStatus,
    db: &Database,
) -> Result<UpdateResult, Error> {
    let query = doc! {"_id":  id, "paymentStatus": { "$ne": "Success" }};
    let update = doc! {"$set": { "paymentStatus": status.to_string()}};
    crud_repository::update_one(query, update, None, COLLECTION_NAME, db).await
}
